@@ -1,5 +1,5 @@
 // Package html minifies HTML5 following the specifications at http://www.w3.org/TR/html5/syntax.html.
-package html // import "github.com/tdewolff/minify/html"
+package html
 
 import (
 	"bytes"
@@ -347,6 +347,10 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 					}
 
 					val := attr.AttrVal
+					if attr.Traits&trimAttr != 0 {
+						val = parse.TrimWhitespace(val)
+						val = parse.ReplaceMultipleWhitespace(val)
+					}
 					if len(val) == 0 && (attr.Hash == html.Class ||
 						attr.Hash == html.Dir ||
 						attr.Hash == html.Id ||
@@ -389,6 +393,7 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 
 					// CSS and JS minifiers for attribute inline code
 					if attr.Hash == html.Style {
+						val = parse.TrimWhitespace(val)
 						attrMinifyBuffer.Reset()
 						if err := m.MinifyMimetype(cssMimeBytes, attrMinifyBuffer, buffer.NewReader(val), inlineParams); err == nil {
 							val = attrMinifyBuffer.Bytes()
@@ -399,6 +404,7 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 							continue
 						}
 					} else if len(attr.Text) > 2 && attr.Text[0] == 'o' && attr.Text[1] == 'n' {
+						val = parse.TrimWhitespace(val)
 						if len(val) >= 11 && parse.EqualFold(val[:11], jsSchemeBytes) {
 							val = val[11:]
 						}
@@ -411,23 +417,26 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 						if len(val) == 0 {
 							continue
 						}
-					} else if len(val) > 5 && attr.Traits&urlAttr != 0 { // anchors are already handled
-						if parse.EqualFold(val[:4], httpBytes) {
-							if val[4] == ':' {
-								if m.URL != nil && m.URL.Scheme == "http" {
-									val = val[5:]
-								} else {
-									parse.ToLower(val[:4])
-								}
-							} else if (val[4] == 's' || val[4] == 'S') && val[5] == ':' {
-								if m.URL != nil && m.URL.Scheme == "https" {
-									val = val[6:]
-								} else {
-									parse.ToLower(val[:5])
+					} else if attr.Traits&urlAttr != 0 { // anchors are already handled
+						val = parse.TrimWhitespace(val)
+						if 5 < len(val) {
+							if parse.EqualFold(val[:4], httpBytes) {
+								if val[4] == ':' {
+									if m.URL != nil && m.URL.Scheme == "http" {
+										val = val[5:]
+									} else {
+										parse.ToLower(val[:4])
+									}
+								} else if (val[4] == 's' || val[4] == 'S') && val[5] == ':' {
+									if m.URL != nil && m.URL.Scheme == "https" {
+										val = val[6:]
+									} else {
+										parse.ToLower(val[:5])
+									}
 								}
+							} else if parse.EqualFold(val[:5], dataSchemeBytes) {
+								val = minify.DataURI(m, val)
 							}
-						} else if parse.EqualFold(val[:5], dataSchemeBytes) {
-							val = minify.DataURI(m, val)
 						}
 					}
 
@@ -441,8 +450,12 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 						if _, err := w.Write(isBytes); err != nil {
 							return err
 						}
+
+						// use double quotes for RDFa attributes
+						isXML := attr.Hash == html.Vocab || attr.Hash == html.Typeof || attr.Hash == html.Property || attr.Hash == html.Resource || attr.Hash == html.Prefix || attr.Hash == html.Content || attr.Hash == html.About || attr.Hash == html.Rev || attr.Hash == html.Datatype || attr.Hash == html.Inlist
+
 						// no quotes if possible, else prefer single or double depending on which occurs more often in value
-						val = html.EscapeAttrVal(&attrByteBuffer, attr.AttrVal, val)
+						val = html.EscapeAttrVal(&attrByteBuffer, attr.AttrVal, val, isXML)
 						if _, err := w.Write(val); err != nil {
 							return err
 						}
