@@ -1,4 +1,4 @@
-package svg // import "github.com/tdewolff/minify/svg"
+package svg
 
 import (
 	"testing"
@@ -11,6 +11,7 @@ func TestPathData(t *testing.T) {
 		pathData string
 		expected string
 	}{
+		{"M10 10 10 10", "M10 10"},
 		{"M10 10 20 10", "M10 10H20"},
 		{"M10 10 10 20", "M10 10V20"},
 		{"M50 50 100 100", "M50 50l50 50"},
@@ -19,12 +20,14 @@ func TestPathData(t *testing.T) {
 		{"M50 50H55V55", "M50 50h5v5"},
 		{"M10 10L11 10 11 11", "M10 10h1v1"},
 		{"M10 10l1 0 0 1", "M10 10h1v1"},
+		{"M10 10v1", "M10 10v1"},
 		{"M10 10L11 11 0 0", "M10 10l1 1L0 0"},
 		{"M246.614 51.028L246.614-5.665 189.922-5.665", "M246.614 51.028V-5.665H189.922"},
 		{"M100,200 C100,100 250,100 250,200 S400,300 400,200", "M1e2 2e2c0-1e2 150-1e2 150 0s150 1e2 150 0"},
 		{"M200,300 Q400,50 600,300 T1000,300", "M2e2 3e2q2e2-250 4e2.0t4e2.0"},
 		{"M300,200 h-150 a150,150 0 1,0 150,-150 z", "M3e2 2e2H150A150 150 0 1 0 3e2 50z"},
 		{"x5 5L10 10", "L10 10"},
+		{"M10 10 10 10", "M10 10"},
 
 		{"M.0.1", "M0 .1"},
 		{"M200.0.1", "M2e2.1"},
@@ -32,6 +35,26 @@ func TestPathData(t *testing.T) {
 		{"A1.1.0.0.0.0.2.3", "A1.1.0.0.0.0.2.3"},                               // bad input (sweep and large-arc are not booleans) gives bad output
 		{"A.0.0.4.0.0.0.3", "A0 0 .4.0.0.0.3"},                                 // bad input, keep dot for booleans
 
+		// change/remove commands
+		{"M10 10L10 10L20 10z", "M10 10H20z"},
+		{"M10 10t10 0", "M10 10h10"},
+		{"M10 10T20 10", "M10 10H20"},
+		{"M10 10T20 10T20 20", "M10 10t10 0 0 10"},
+		{"M10 10q0 0 10 0", "M10 10h10"},
+		{"M10 10Q10 10 20 10", "M10 10H20"},
+		{"M10 10Q20 10 20 10", "M10 10H20"},
+		{"M10 10Q15 20 20 10Q25 0 30 10", "M10 10q5 10 10 0t10 0"},
+		{"M10 10s0 0 10 0", "M10 10h10"},
+		{"M10 10S10 10 20 10", "M10 10H20"},
+		{"M10 10S20 10 20 10", "M10 10H20"},
+		{"M10 10c0 0 0 0 10 0", "M10 10h10"},
+		{"M10 10C10 10 10 10 20 10", "M10 10H20"},
+		{"M10 10C10 10 20 10 20 10", "M10 10H20"},
+		{"M10 10C20 10 10 10 20 10", "M10 10H20"},
+		{"M10 10C20 10 20 10 20 10", "M10 10H20"},
+		{"M10 10C10 20 20 20 20 10C20 0 30 0 30 10", "M10 10c0 10 10 10 10 0S30 0 30 10"},
+		// does not have C/S -> Q/T since subsequent commands could rely on the control points, and both are incompatible for that
+
 		// fuzz
 		{"", ""},
 		{"ML", ""},
@@ -60,6 +83,7 @@ func TestPathDataTruncated(t *testing.T) {
 		expected string
 	}{
 		{"m100 0 50 50zM100 0z", "m1e2.0 50 50zm0 0z"},
+		{"M194.4 16.4C194.4 7.4 187 0 177.9 0 168.8 0 161.5 7.4 161.5 16.4", "M194.4 16.4c0-9-7.4-16.4-16.5-16.4s-16.4 7.4-16.4 16.4"}, // #233
 	}
 
 	p := NewPathData(&Minifier{Decimals: 3})
