@@ -1,4 +1,4 @@
-package svg // import "github.com/tdewolff/minify/svg"
+package svg
 
 import (
 	"bytes"
@@ -29,6 +29,7 @@ func TestSVG(t *testing.T) {
 		{`<svg version="1.0"></svg>`, `<svg version="1.0"/>`},
 		{`<svg version="1.1" x="0" y="0px" width="100%" height="100%"><path/></svg>`, `<svg><path/></svg>`},
 		{`<path x="a"> </path>`, `<path x="a"/>`},
+		{`<path x=""> </path>`, `<path/>`},
 		{`<path x=" a "/>`, `<path x="a"/>`},
 		{"<path x=\" a \n b \"/>", `<path x="a b"/>`},
 		{`<path x="5.0px" y="0%"/>`, `<path x="5" y="0"/>`},
@@ -48,14 +49,10 @@ func TestSVG(t *testing.T) {
 		{`<path fill="#fff"/>`, `<path fill="#fff"/>`},
 		{`<path fill="white"/>`, `<path fill="#fff"/>`},
 		{`<path fill="#ff0000"/>`, `<path fill="red"/>`},
-		{`<line x1="5" y1="10" x2="20" y2="40"/>`, `<path d="M5 10 20 40z"/>`},
-		{`<rect x="5" y="10" width="20" height="40"/>`, `<path d="M5 10h20v40H5z"/>`},
-		{`<rect x="-5.669" y="147.402" fill="#843733" width="252.279" height="14.177"/>`, `<path fill="#843733" d="M-5.669 147.402h252.279v14.177H-5.669z"/>`},
-		{`<rect x="5" y="10" rx="2" ry="3"/>`, `<rect x="5" y="10" rx="2" ry="3"/>`},
+		{`<rect x="5" y="10" rx="2" ry="3">`, ``},
 		{`<rect x="5" y="10" height="40"/>`, ``},
-		{`<rect x="5" y="10" width="30" height="0"/>`, ``},
-		{`<polygon points="1,2 3,4"/>`, `<path d="M1 2 3 4z"/>`},
-		{`<polyline points="1,2 3,4"/>`, `<path d="M1 2 3 4"/>`},
+		{`<rect x="5" y="10" width="30" height="0%"/>`, ``},
+		{`<rect x="5" y="10" width="30%" height="100%"/>`, `<rect x="5" y="10" width="30%" height="100%"/>`},
 		{`<svg contentStyleType="text/json ; charset=iso-8859-1"><style>{a : true}</style></svg>`, `<svg contentStyleType="text/json;charset=iso-8859-1"><style>{a : true}</style></svg>`},
 		{`<metadata><dc:title /></metadata>`, ``},
 
@@ -64,6 +61,9 @@ func TestSVG(t *testing.T) {
 
 		{`<polygon fill="none" stroke="#000" points="-0.1,"/>`, `<polygon fill="none" stroke="#000" points="-0.1,"/>`}, // #45
 		{`<path stroke="url(#UPPERCASE)"/>`, `<path stroke="url(#UPPERCASE)"/>`},                                       // #117
+		{`<rect height="10"/><path/>`, `<path/>`},                                                                      // #244
+		{`<rect height="10"><path/></rect>`, ``},                                                                       // #244
+		{`<rect height="10"><path/></g>`, ``},                                                                          // #244
 
 		// go fuzz
 		{`<0 d=09e9.6e-9e0`, `<0 d=""`},
